# -*- coding: utf-8 -*-
"""
    Copyright (C) 2023  Anders Håål

    This file is part of infoblox-discovery.

    infoblox-discovery is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    infoblox-discovery is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with infoblox-discovery.  If not, see <http://www.gnu.org/licenses/>.

"""

DISCOVERY_CONFIG = 'INFOBLOX_DISCOVERY_CONFIG'
DISCOVERY_PROMETHEUS_SD_FILE_DIRECTORY = 'INFOBLOX_DISCOVERY_PROMETHEUS_SD_FILE_DIRECTORY'
DISCOVERY_HOST = 'INFOBLOX_DISCOVERY_HOST'
DISCOVERY_PORT = 'INFOBLOX_DISCOVERY_PORT'
DISCOVERY_BASIC_AUTH_USERNAME = 'INFOBLOX_DISCOVERY_BASIC_AUTH_USERNAME'
DISCOVERY_BASIC_AUTH_PASSWORD = 'INFOBLOX_DISCOVERY_BASIC_AUTH_PASSWORD'
DISCOVERY_BASIC_AUTH_ENABLED = 'INFOBLOX_DISCOVERY_BASIC_AUTH_ENABLED'
DISCOVERY_LOG_FILE = 'INFOBLOX_DISCOVERY_LOG_FILE'
DISCOVERY_LOG_LEVEL = 'INFOBLOX_DISCOVERY_LOG_LEVEL'
DISCOVERY_CACHE_TTL = 'INFOBLOX_DISCOVERY_CACHE_TTL'
DISCOVERY_FETCH_INTERVAL = 'INFOBLOX_DISCOVERY_FETCH_INTERVAL'
